// iob_com_test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>
#include <windef.h>
#include <winnt.h>

#import "RPI_IOBoard.tlb" raw_interfaces_only
using namespace RPI_IOBoard;

void iob_test();


int _tmain(int argc, _TCHAR* argv[])
{
	iob_test();
	while(1) {}
	return 0;
}

void iob_test()
{
	// initialize COM
	HRESULT hr = CoInitialize(NULL);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't initialize COM, HRESULT=%d\n", hr);
	} else {
		printf("success: initialized COM\n");
	}

	// create COM instance
	ICoreFunctions *pICoreFunctions = NULL;
	hr = CoCreateInstance( __uuidof(IOBoard), NULL, CLSCTX_INPROC_SERVER, __uuidof(ICoreFunctions), (void**) &pICoreFunctions );
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't create instance of COM class, HRESULT=%d\n", hr);
		return;
	} else {
		printf("success: created instance of COM class\n");
	}


	// try to connect to board

	long numDevices;
	hr = pICoreFunctions->getNumDevices(&numDevices);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't getNumDevices(), HRESULT=%d\n", hr);
		exit(0);
	} else {
		printf("success: getNumDevices() found %d devices\n", numDevices);
	}
	if (numDevices < 1) exit(0);

	BSTR msg;
	hr = pICoreFunctions->openDevice(1, &msg);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't openDevice(), HRESULT=%d\n", hr);
		return;
	} else {
		printf("success: openDevice() returned '%s'\n", (char *)_bstr_t(msg));
	}

	// properties need to be accessed using corresponding get_*() or set_*() function
	// e.g., .FirmwareVersion in MATLAB or Python code becomes get_FirmwareVersion() here
	hr = pICoreFunctions->get_FirmwareVersion(&msg);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't get_FirmwareVersion(), HRESULT=%d\n", hr);
		return;
	} else {
		printf("success: get_FirmwareVersion() returned '%s'\n", (char *)_bstr_t(msg));
	}


	// acquire IDigitalIO interface and call a function on it
	IDigitalIO *pIDigitalIO = NULL;
	hr = pICoreFunctions->QueryInterface(__uuidof(IDigitalIO), (void**) &pIDigitalIO);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't get IDigitalIO interface, HRESULT=%d\n", hr);
		return;
	} else {
		printf("success: got IDigitalIO interface\n");
	}

	VARIANT_BOOL direction;
	hr = pIDigitalIO->getDigitalDirection(1, &direction);
	if (!SUCCEEDED(hr)) {
		fprintf(stderr, "error: couldn't getDigitalDirection(), HRESULT=%d\n", hr);
		return;
	} else {
		// 0 for output, 1 for input
		printf("success: getDigitalDirection() returned '%d'\n", direction);
	}

}
